package hicosim.components.backend.processor;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import hicosim.components.backend.io.ObjectfileReader;
import hicosim.control.Engine;


/**
 * Class which represents the memory.
 * @author Michael Wager, Luca Calchera,Weber 16th May 2009
 *
 */
public class Memory implements IMemory {
	
	private IPluginManager pluginManager;
	
	private SortedMap<Integer, Integer> memory;

	private Labels labels=null;	
	private Engine engine=null;
	
	private Disassembler disas=null;
		
	private HashSet<Integer> watchPoints = new HashSet<Integer>();
		
	/**
	 * Constructor - set up TreeMap
	 * @param  - Engine reference
	 */
	public Memory(Engine engine) {	//map muss sortiert sein!!!
		this.memory = Collections.synchronizedSortedMap(new TreeMap<Integer, Integer>());
		
		/* Memory instanziiert neuen PluginManager und uebergibt sich selbst als
		 * Memory */
		pluginManager = new PluginManager(this);
		this.engine = engine;

		//########   HIER DEN DISASSEMBLER INSTANZIEREN:
		this.disas = new Disassembler(this, this.engine);//braucht memory und engine

	}
	
	public void resetMemory() {
		this.memory.clear();//???
		this.pluginManager.reFreshPlugins();
		//also delete watchpoints:
		this.watchPoints=new HashSet<Integer>();
	}
	/**
	 * This method is used to read the value at the given address in decimal.
	 */
	public int readValueAt(int address) {
		try {			
			return this.memory.get(address);
		}
		catch (NullPointerException e) {
			return 0;
		}
		catch (Exception ex) {
			return 0;
		}
	}

	public String getDisassembledInstructionAt(int address) throws Exception {
		try {
			return this.disas.getDisassembledInstructionAt(address);
		}
		catch (NullPointerException e) {
			return null;
		}
	}
	/**
	 * This method is used to write the value at the given address.
	 */
	public void writeTo(int address, int value) {
		this.memory.put(address, value);
		
//		disasembler update:
		try {
			disas.putInstr(address, Disassembler.disasOneSingleInstruction(address, value));
		}
		catch (Exception e) {
			disas.putInstr(address, "NONE");
		}
		
		/* PluginManager prueft, ob die beschriebene Speicherzelle
		 * von einem Plugin ueberwacht wird
		 */
		pluginManager.notifyPlugins(address, value);
		
		/* Check if a watchpoint is set on given address */
		if(watchPoints.contains((Integer)address)){
			engine.displayConsole("### Watchpoint reached at address " + address, false);
			
			//highlight!
			int ip = engine.getIP();
			if(engine.getSourcefile().getLines() != null)//eine Zeile nachher, da Ausfuehrung schon durchgefuehrt wurde!
				engine.highlightSourcecodeline(engine.getSourcefile().getLineNumberAtAddress( ip ) );
			//eine Zeile nachher, da Ausfuehrung schon durchgefuehrt wurde!
			
			engine.highlightDisascodeline(this.disas.getLineOfAddress( ip + 1 ));
			
			try {
				this.engine.getProcessor().interrupt();
			} catch (Exception e) {
				this.engine.displayConsole("Processor interrupt failed while watchpoint reached at address " + address, true);
//				e.printStackTrace();
			}
		}
	}
	
	/**
	 * This method reads in the objectfile and sets up all the relevant information,
	 * like memory, processor, progCounter, Labels
	 *
	 * @param PATH - PATH to the objectFile
	 * @return - InfoString == "" if everything is OK.
	 * @throws Exception
	 */
	public String loadObjectfile(String PATH) throws Exception {
		//loading Objectfile	

		//IMMER NEU INSTANZIEREN!!!
		ObjectfileReader reader = new ObjectfileReader(this);
		int a=0;
		try {
			a = reader.setUp(PATH); //vorer setUp Path!int a = this.reader.setUp(PATH); 
		}
		catch (Exception e) {
			throw new Exception(PATH + e.getMessage());
		}

		if (a<0){ //setUp gibt -1 wenn was nicht stimmt
			throw new Exception(PATH + " -> NO SUCH FILE!");
		}
		else {
			//=============  RESET VORHER:
			this.engine.reset();
			this.engine.resetMemory();
			this.engine.getProcessor().reset();
			
			//ZUERST SHORTS LESEN UND startAddressen fuer HiCoVec-Befehle  berechnen
			// um dann in readBtes auszuwerten! (siehe globale variablen "start-/endAddressIndex" im reader)
			//...auch Speicherinhalt berechnen:
			reader.readUnsignedShortsAndGetMemory();

			//nochmal setup
			reader.setUp(PATH);  //hier Abfrage des return werts nich noetig

			//die fuer hicovec/sim relevanten 32 bit Befehle:
			reader.readUnsignedBytes();

			//close Streams !!!
			reader.closeStreams();		
			
			//...labels
			this.labels = reader.getLabels();
						
			//...IP
			int programCounter = reader.getProgramCounter();

			// nach jedem laden eines Object-Files: Alle Plugins refreshen
			this.pluginManager.reFreshPlugins();
	
			/* 
			 * ##### set up cpu: --> memory and prog counter #####
			 */
			this.engine.getProcessor().setUp(this, programCounter);

			//DISAS UND DEN GEFUELLTEN SPEICHER AUF GUI ANZEIGEN:
			if(this.engine.getListener()!=null) {
				this.engine.getListener().displayDisassembledCode(this.disas.disas());
			}
			
			//all fine
			return ""; 
		}
	}
	
	/**
	 * This method returns the disassembler reference.
	 */
	public Disassembler getDisassembler() {
		if(this.disas!=null) 
			return this.disas;
		else
			return null;
	}
	/**
	 * This method returns all addresses of the memory.
	 */
	public Set<Integer> getAllUsedAddresses() {
		return this.memory.keySet();
	}
	
	/**
	 * String representation of the memory.
	 */
	public String toString() {
		return this.memory.toString();
	}
	/**
	 * This method returns the labels.
	 */
	public Labels getLabels() {
		return this.labels;
	}
	/**
	 * This method is used to get the label at the given address.
	 */
	public String getLabelAt(int address) {
		try {
			return this.labels.getLabelAt(address);
		}
		catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * This method is used to get the address of a label.
	 */
	public int getAddressOfLabel(String label) {
		return this.labels.getAddressOfLabel(label);
	}
	
	/* ab hier watchpoints, muss noch kommentiert werden */
	public void setWatchPoint(int adr){
		
		if(watchPoints.contains(adr)){
			this.engine.displayConsole("Watchpoint already set at " + adr, true);
		} else {
			watchPoints.add(adr);
			this.engine.displayConsole("Watchpoint set at " + adr, false);
			pluginManager.notifyPlugins(adr, readValueAt(adr));
		}
	}
	
	public void delWatchPoint(int adr){
		if(watchPoints.contains((Integer)adr)){
			watchPoints.remove((Integer)adr);
			this.engine.displayConsole("Watchpoint removed from address " + adr, false);
			pluginManager.notifyPlugins(adr, readValueAt(adr));
		} else {
			this.engine.displayConsole("No watchpoint set at given address " + adr, true);
		}
	}
	
	public boolean isWatchPoint(int adr){
		return watchPoints.contains((Integer)adr);
	}
	
	public HashSet<Integer> getAllWatchPoints () {
		return this.watchPoints;
	}
	
	public IPluginManager getPluginManager(){
		return pluginManager;
	}
	
}
