package hicosim.components.frontend;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.LinkedList;

import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;

/**
 * This class extends the super class JTextArea and appends to each line of text
 * the corresponding line number. It also provides methods to set and remove
 * break points. Moreover it provides a method to highlight single lines of
 * code.
 *
 * @author Daniel Obermueller & Sebastian Minning 28th April 2009
 *
 */
public class JTextAreaWithLineNumbering extends JTextArea {
	private static final long serialVersionUID = 1L;
	private LinkedList<Integer> breakpoints;
	final int SPACE = getFontMetrics(getFont()).stringWidth(
			"                    ");

	public static void main(String[] args) throws InterruptedException {
		javax.swing.JFrame frame = new javax.swing.JFrame();
		frame.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent ev) {
				System.exit(0);
			}
		});

		java.awt.Container contentPane = frame.getContentPane();
		javax.swing.JPanel subpanel;
		JTextAreaWithLineNumbering lnp;

		lnp = new JTextAreaWithLineNumbering(10, 10);

		subpanel = new javax.swing.JPanel(new java.awt.BorderLayout());
		subpanel.add(new javax.swing.JScrollPane(lnp),
				java.awt.BorderLayout.CENTER);
		contentPane.add(subpanel);
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < 1000000; i++)
			buffer.append(i + "\n");

		lnp.setText(buffer.toString());
		frame.setSize(800, 600);
		frame.show();
		Thread.sleep(1000);
		lnp.highlightLine(0);
		lnp.setBreakpointMark(0);
		lnp.setBreakpointMark(10);
		lnp.setBreakpointMark(999999);
		Thread.sleep(1000);
		lnp.deleteBreakpointMark(10);
		lnp.deleteBreakpointMark(10);
		lnp.highlightLine(5);
	}

	public JTextAreaWithLineNumbering(int rows, int cols) {
		super(rows, cols);
		setOpaque(false);
		breakpoints = new LinkedList<Integer>();
	}

	/**
	 * This method is an overwritten method of the super class and returns the
	 * insets of the JTextArea.
	 *
	 * @author Daniel Obermueller & Sebastian Minning 28th April 2009
	 * @return Insets
	 *
	 */
	public Insets getInsets() {
		return getInsets(new Insets(0, 0, 0, 0));
	}

	/**
	 * This method modifies the insets, by adding space for the line number on
	 * the left side.
	 *
	 * @author Daniel Obermueller & Sebastian Minning 28th April 2009
	 * @return Insets
	 * @param Insets
	 *            insets
	 */
	public Insets getInsets(Insets insets) {
		insets = super.getInsets(insets);
		insets.left += SPACE;
		return insets;
	}

	/**
	 * This method is painting the JTextArea and is always invoked, when the
	 * text field is edited.
	 *
	 * @author Daniel Obermueller & Sebastian Minning 28th April 2009
	 * @param Graphics
	 *            g
	 */
	public void paintComponent(Graphics g) {
		Insets insets = getInsets();
		Rectangle clip = g.getClipBounds();
		g.setColor(getBackground());
		g.fillRect(clip.x, clip.y, clip.width, clip.height);
		/* If the line numbering needs to be repainted. */
		if (clip.x < insets.left) {
			FontMetrics fontMetrics = g.getFontMetrics();
			int fontHeight = fontMetrics.getHeight();
			/*
			 * "y" is initialized and represents the top of the JTextArea text
			 * field.
			 */
			int currentY = fontMetrics.getAscent() + insets.top;
			/*
			 * Determine the first line number, which needs to be painted.
			 */
			int currentLineNumber = ((clip.y + insets.top) / fontHeight) + 1;
			/*
			 * If the painting of the line numbers is not starting with 1,
			 * determine the top of the current section of the JTextArea text
			 * field.
			 */
			if (currentY < clip.y) {
				currentY = currentLineNumber * fontHeight
						- (fontHeight - fontMetrics.getAscent());
			}
			int endingPointY = currentY + clip.height + fontHeight;
			/*
			 * "x" is initialized and represents the left bound of the JTextArea
			 * text field.
			 */
			int startingPointX = insets.left - SPACE;
			g.setColor(getForeground());
			/*
			 * Paint the line numbers of the current section of the JTextArea
			 * text field.
			 */
			while (currentY < endingPointY) {
				String index = appendBlanks(currentLineNumber - 1);
				if (breakpoints.contains(currentLineNumber - 1)) {
					g.drawString(index + Character.toString((char) 8226),
							startingPointX, currentY);
				} else
					g.drawString(index, startingPointX, currentY);
				currentY += fontHeight;
				currentLineNumber++;
			}
		}
		super.paintComponent(g);
	}

	/**
	 * This method returns an ArrayList<Integer>, which contains the length of
	 * each line. Each length of a line can be accessed by invoking the get(int
	 * index) method of the ArrayList<Integer>. (This method is used by the
	 * method "highlightLine(int index).)
	 *
	 * @author Daniel Obermueller 28th April 2009
	 * @return ArrayList<Integer>
	 */
	private ArrayList<Integer> indexText() {
		try {
			String text = getDocument().getText(
					getDocument().getStartPosition().getOffset(),
					getDocument().getEndPosition().getOffset());

			String[] lines = text.split("\n");
			ArrayList<Integer> indexedText = new ArrayList<Integer>();
			for (int i = 0; i < lines.length; i++) {
				String line = lines[i] + "\n";
				/*
				 * If the current line is not the first line of the JTextArea,
				 * store the sum of the length of the current line, which is the
				 * length of the whole text yet and the length of the last line.
				 */
				if (indexedText.size() != 0) {
					int lastIndex = indexedText.get(indexedText.size() - 1);
					indexedText.add(lastIndex + line.length());
				}
				/*
				 * If the current line is the first line of the JTextArea, store
				 * the length of it.
				 */
				else
					indexedText.add(line.length());
			}
			return indexedText;
		} catch (BadLocationException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * This method converts a passed Integer-Object into a String-Object and
	 * appends a defined amount of blanks to it, eventually it returns the
	 * String-Object.
	 *
	 * @author Daniel Obermueller 28th April 2009
	 * @return String
	 * @param int
	 *            i
	 */
	private String appendBlanks(int i) {
		final int AMOUNTOFBLANKS = 8;
		String index = "" + (i + 1);
		for (int j = 0; j < (AMOUNTOFBLANKS + 1
				- (String.valueOf((i + 1)).length()) - 1); j++)
			index = " " + index;
		index += "\n";
		return index;
	}

	/**
	 * This method highlights a line, which is specified by the passed
	 * Integer-Object "index", by accessing inherited methods, which provide
	 * several options to highlight a text.
	 *
	 * @author Daniel Obermueller 28th April 2009
	 * @param int
	 *            index
	 */
	public void highlightLine(int index) {
		final Highlighter.HighlightPainter HIGHLIGHTER = new DefaultHighlighter.DefaultHighlightPainter(
				Color.RED);
		ArrayList<Integer> indexes = indexText();
		try {
			int startOffset;
			int endOffset = indexes.get(index) - 1;
			/*
			 * If the passed line is the first line of the JTextArea, start with
			 * 0.
			 */
			if (index == 0)
				startOffset = 0;
			/*
			 * Else retrieve the proper starting point.
			 */
			else
				startOffset = indexes.get(index - 1);
			getHighlighter().removeAllHighlights();
			setCaretPosition(startOffset);
			getHighlighter().addHighlight(startOffset, endOffset, HIGHLIGHTER);
		} catch (IndexOutOfBoundsException e) {
			throw new IndexOutOfBoundsException(index
					+ " exceeds the bounds of the ArrayList!");
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method displays a break point mark, which is specified by the passed
	 * Integer-Object "index".
	 *
	 * @author Daniel Obermueller 28th April 2009
	 * @param int
	 *            index
	 */
	public void setBreakpointMark(int index) {
		/* If the passed break point already exists, do nothing. */
		if (breakpoints.contains(index))
			return;
		/* Else add the break point to the list of break points. */
		breakpoints.add(index);
		repaint();
	}

	/**
	 * This method deletes a break point mark, which is specified by the passed
	 * Integer-Object "index".
	 *
	 * @author Daniel Obermueller 28th April 2009
	 * @param int
	 *            index
	 */
	public void deleteBreakpointMark(int index) {
		/* If the break point exists, it will be removed, else nothing happens. */
		
		breakpoints.remove(new Integer(index));//.removeFirstOccurrence(new Integer(index));//KLAPPT AUF MAC AUCH NICHT!!!
		repaint();
	}

	/**
	 * This method deletes all break points.
	 *
	 * @author Daniel Obermueller 28th April 2009
	 */
	public void deleteAllBreakpointMarks() {
		breakpoints = new LinkedList<Integer>();
		repaint();
	}

}