----------------------------------------------------------------------------------
-- Company:        Fachhochschule Augsburg - Fakultaet fuer Informatik
-- Engineer:       Schaeferling Michael
-- 
-- Create Date:    16:22:20 05/06/2007 
-- Design Name: 
-- Module Name:    debugger - RTL
-- Project Name: 
-- Target Devices: Xilinx
-- Tool versions: 
-- Description: 	 This module provides debugging operation
--
-- Dependencies: 
--
-- Revision: 0.51
-- Additional Comments: 
-- Last Change: 09-03-23
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity debugger is
	port(
		-- Clock and SerialPort
  		CLK_50MHZ:		in std_logic;
		RS232_RXD:		in std_logic;
		RS232_TXD:		out std_logic;

		-- Internal Chip Debugging Connectors
		-- clocks off scan path
		NOSCAN_CLK:		out std_logic_vector(3 downto 0)	:= "0000";
		-- clocks on scan path
	   SCAN_CLK:		out std_logic_vector(3 downto 0)	:= "0000";
		SCAN_ENABLE:	out std_logic								:= '0';
		-- data to/from chain
		CHAIN_OUT:		out std_logic_vector(3 downto 0)	:= "0000";
		CHAIN_IN:		in std_logic_vector(3 downto 0)
	);
end debugger;

architecture RTL of debugger is

-- States and SubStates
type DEBUG_STATES is (	IDLE, 
								STEP_CHAINMASK_0, STEP_CHAINMASK_1, STEP_STEPCOUNT_0, STEP_STEPCOUNT_1, 
									STEP_ASSURED, STEP_CLOCKING_CLK1, STEP_CLOCKING_CLK0, STEP_WAIT_RS232, 
								RUN_CHAINMASK_0, RUN_CHAINMASK_1, RUN_ASSURED, 
								   RUN_CLOCKING_CLK0, RUN_CLOCKING_CLK1, RUN_WAIT_RS232, 
								WRITECHAIN_LENGTH_0, WRITECHAIN_LENGTH_1, WRITECHAIN_ASSURED, WRITECHAIN_DATA, 
									WRITECHAIN_CLK1, WRITECHAIN_CLK0, WRITECHAIN_WAIT_RS232,
								READCHAIN_LENGTH_0, READCHAIN_LENGTH_1, READCHAIN_ASSURED, 
									READCHAIN_CLK1, READCHAIN_CLK0, 
									READCHAIN_SENDBITS, READCHAIN_CHECKSUM
								);
signal DEBUG_STATE: DEBUG_STATES := IDLE;

-- local RS232 signals
signal DATA_IN, DATA_OUT:  std_logic_vector(7 downto 0) := (others => '0');
signal TX_SEND_DATA, RX_DATA_RCVD: std_logic := '0';
signal TX_BUSY, RX_BUSY : std_logic := '0';
signal CHAIN_WRITE_ENABLE : std_logic := '0';
signal CHAIN_WR_OUT : std_logic_vector (3 downto 0) := (others => '0');

-- was variable, now is signal:
signal CHAIN_DATA: std_logic_vector(6 downto 0) := (others => '0');

-- 
signal send_alive: std_logic := '0';
signal checksum: std_logic_vector(7 downto 0) := (others => '0');


component rs232
	generic(
			DATABITS		: integer:= 8;
			STARTBITS	: integer:= 1;
			STOPBITS		: integer:= 1
	);
	port(
			CLK_50MHZ	: in std_logic;
			RS232_RXD	: in std_logic;
			RS232_TXD	: out std_logic;

			-- Data to send
			DATA_TX		: in std_logic_vector(DATABITS-1 downto 0);
			TX_SEND_DATA	: in std_logic;
			TX_BUSY		: out std_logic;

			-- Data to receive
			DATA_RX		: out std_logic_vector(DATABITS-1 downto 0);
			RX_DATA_RCVD	: out std_logic;
			RX_BUSY		: out std_logic
	);
end component;
  
for SER_0: rs232 use entity WORK.rs232(RTL);


begin
-- port mapping
SER_0: rs232 port map(
		CLK_50MHZ => CLK_50MHZ, 
		RS232_RXD => RS232_RXD, 
		RS232_TXD => RS232_TXD,
		DATA_TX => DATA_OUT, 
		DATA_RX => DATA_IN,
		TX_SEND_DATA => TX_SEND_DATA, 
		RX_DATA_RCVD => RX_DATA_RCVD,
		TX_BUSY => TX_BUSY, 
		RX_BUSY => RX_BUSY
	);

-- logic


-- "i'm alive" process
process (CLK_50MHZ)
constant ALIVE_PERIOD:	std_logic_vector(23 downto 0) := x"4C4B40"; -- send alive-byte every 10th of a second
variable alive_cnt: 	std_logic_vector(23 downto 0) := ALIVE_PERIOD;
begin
	if CLK_50MHZ'event AND CLK_50MHZ='1' then
		send_alive <= '0';
		alive_cnt := alive_cnt - 1;
		
		if ((alive_cnt = 0) OR (TX_BUSY = '1')) then
			alive_cnt := ALIVE_PERIOD;
			if (TX_BUSY = '0' and RX_BUSY = '0') then
				send_alive <= '1';
			end if;
		end if;
		
	end if;
end process;


process (CHAIN_IN, CHAIN_WR_OUT, CHAIN_WRITE_ENABLE)
begin
  if (CHAIN_WRITE_ENABLE = '1') then
    CHAIN_OUT <= CHAIN_WR_OUT;
  else
    CHAIN_OUT <= CHAIN_IN;
  end if;
end process;


P_STATE_CHG: process (CLK_50MHZ)

variable STEPCOUNT:          std_logic_vector(13 downto 0) := (others => '0');
variable STEPMASK:           std_logic_vector(7 downto 0)  := (others => '0');

variable RUNMASK:            std_logic_vector(7 downto 0)  := (others => '0');

variable CHAIN_LENGTH:       std_logic_vector(13 downto 0) := (others => '0');
variable CHAIN_ID:           std_logic_vector(1 downto 0)  := (others => '0');
variable CHAIN_SINGLEBYTE:   std_logic_vector(2 downto 0)  := (others => '0');

-- Acknowledge_Words
constant ACK_ALIVE:				std_logic_vector(7 downto 0)  := x"41"; -- 01000001
constant ACK_DONE:				std_logic_vector(7 downto 0)  := x"4f"; -- 01001111

constant INS_STOP:				std_logic_vector(7 downto 0)  := x"05"; -- 00000101

begin
	if CLK_50MHZ'event AND CLK_50MHZ='1' then
		
		NOSCAN_CLK(3 downto 0)	<= "0000";										
		SCAN_CLK(3 downto 0)		<= "0000";
		SCAN_ENABLE						<= '0';
		
		--CHAIN_OUT (3 downto 0)	<= "0000";
		TX_SEND_DATA					<= '0';
		
		CHAIN_WRITE_ENABLE 			<= '0';
		CHAIN_WR_OUT 					<= "0000";
		
		case (DEBUG_STATE) is
		----- IDLE 
			when IDLE =>		 			if RX_DATA_RCVD = '1' then
													if DATA_IN(7) = '0' then -- INSTRUCTION
														checksum <= DATA_IN;
														
														if DATA_IN(1 downto 0) = "01" then 
														   if DATA_IN(3 downto 2) = "00" then     -- Step
															   DEBUG_STATE <= STEP_CHAINMASK_0;
															elsif DATA_IN(3 downto 2) = "11" then  -- Run
																DEBUG_STATE <= RUN_CHAINMASK_0;
														   else DEBUG_STATE <= IDLE;
														   end if;
														
														elsif DATA_IN(1 downto 0) = "10" then
															CHAIN_ID := DATA_IN(6 downto 5);
															if DATA_IN(3 downto 2) = "11" then     -- WriteChain
																DEBUG_STATE <= WRITECHAIN_LENGTH_0;
															elsif DATA_IN(3 downto 2) = "01" then  -- ReadChain
																DEBUG_STATE <= READCHAIN_LENGTH_0;
															else DEBUG_STATE <= IDLE;
															end if;
														
														end if;
													else
														DEBUG_STATE <= IDLE;
													end if;
												
												elsif ((send_alive = '1') AND (TX_BUSY = '0')) then
													DATA_OUT <= ACK_ALIVE;
													TX_SEND_DATA <= '1';
													
												end if;
											
		----- RUN / STOP
			when RUN_CHAINMASK_0 =>	if RX_DATA_RCVD = '1' then
													RUNMASK(3 downto 0) := DATA_IN(3 downto 0); -- noScanPath
													checksum <= checksum xor DATA_IN;
													DEBUG_STATE <= RUN_CHAINMASK_1;
												end if;

			when RUN_CHAINMASK_1 => 	if RX_DATA_RCVD = '1' then
													RUNMASK(7 downto 4) := DATA_IN(3 downto 0); -- ScanPath
													checksum <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
													DATA_OUT <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
													TX_SEND_DATA <= '1';
													DEBUG_STATE <= RUN_ASSURED;
												end if;

			when RUN_ASSURED =>			if RX_DATA_RCVD = '1' then
													if (DATA_IN = checksum) then
														DEBUG_STATE <= RUN_CLOCKING_CLK0;
													else
														DEBUG_STATE <= IDLE;
													end if;
												end if;

			when RUN_CLOCKING_CLK1 =>	if ((RX_DATA_RCVD = '1') AND (DATA_IN = INS_STOP)) then -- stop instruction recvd
													if TX_BUSY = '1' then
														DEBUG_STATE <= RUN_WAIT_RS232;
													else
														DATA_OUT <= ACK_DONE;
														TX_SEND_DATA <= '1';
														DEBUG_STATE <= IDLE;
													end if;
												else
													-- give the specific clocking to the domains
													NOSCAN_CLK(3 downto 0) <= RUNMASK(3 downto 0);
													SCAN_CLK(3 downto 0) <= RUNMASK(7 downto 4);
													DEBUG_STATE <= RUN_CLOCKING_CLK0;
												end if;
												
			when RUN_CLOCKING_CLK0 =>	if ((RX_DATA_RCVD = '1') AND (DATA_IN = INS_STOP)) then -- got stop instruction
													if TX_BUSY = '1' then
														DEBUG_STATE <= RUN_WAIT_RS232;
													else
														DATA_OUT <= ACK_DONE;
														TX_SEND_DATA <= '1';
														DEBUG_STATE <= IDLE;
													end if;
												else
													-- clocks set to 0 by default
													DEBUG_STATE <= RUN_CLOCKING_CLK1;
												end if;
												
			when RUN_WAIT_RS232		=> if (TX_BUSY = '0') then
													DATA_OUT <= ACK_DONE;
													TX_SEND_DATA <= '1';
													DEBUG_STATE <= IDLE;
												end if;
											
		----- STEP 
			when STEP_CHAINMASK_0 =>	if RX_DATA_RCVD = '1' then
													checksum <= checksum xor DATA_IN;
													STEPMASK(3 downto 0) := DATA_IN(3 downto 0); -- noScanPath
													DEBUG_STATE <= STEP_CHAINMASK_1;
												end if;
												
			when STEP_CHAINMASK_1 => 	if RX_DATA_RCVD = '1' then
													checksum <= checksum xor DATA_IN;
													STEPMASK(7 downto 4) := DATA_IN(3 downto 0); -- ScanPath
													DEBUG_STATE <= STEP_STEPCOUNT_0;
												end if;
												
			when STEP_STEPCOUNT_0 =>	if RX_DATA_RCVD = '1' then
													checksum <= checksum xor DATA_IN;
													STEPCOUNT(6 downto 0) := DATA_IN(6 downto 0);
													DEBUG_STATE <= STEP_STEPCOUNT_1;
												end if;
												
			when STEP_STEPCOUNT_1 =>	if RX_DATA_RCVD = '1' then
													STEPCOUNT(13 downto 7) := DATA_IN(6 downto 0);
													checksum <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
													DATA_OUT <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
													TX_SEND_DATA <= '1';
													DEBUG_STATE <= STEP_ASSURED;
												end if;

			when STEP_ASSURED =>			if RX_DATA_RCVD = '1' then
													if (DATA_IN = checksum) then
														DEBUG_STATE <= STEP_CLOCKING_CLK1;
													else
														DEBUG_STATE <= IDLE;
													end if;
												end if;

			when STEP_CLOCKING_CLK1 =>	-- clocking paths as defined in the masks
												if STEPCOUNT > "00000000000000" then
													STEPCOUNT := STEPCOUNT - "00000000000001";
													NOSCAN_CLK(3 downto 0) <= STEPMASK(3 downto 0);
													SCAN_CLK(3 downto 0) <= STEPMASK(7 downto 4);
													DEBUG_STATE <= STEP_CLOCKING_CLK0;
												else												
													if TX_BUSY = '1' then
														DEBUG_STATE <= STEP_WAIT_RS232;
													else
														DATA_OUT <= ACK_DONE;
														TX_SEND_DATA <= '1';
														DEBUG_STATE <= IDLE;
													end if;
												end if;
												
			when STEP_CLOCKING_CLK0 =>	-- by default clocking paths with '0'
												DEBUG_STATE <= STEP_CLOCKING_CLK1;
												
			when STEP_WAIT_RS232		=> if TX_BUSY = '0' then
													-- send ack for clocking being done:
													DATA_OUT <= ACK_DONE;
													TX_SEND_DATA <= '1';
													DEBUG_STATE <= IDLE;
												end if;
									
		----- WRITECHAIN
			when WRITECHAIN_LENGTH_0 =>	if RX_DATA_RCVD = '1' then
														checksum <= checksum xor DATA_IN;
														DEBUG_STATE <= WRITECHAIN_LENGTH_1;
														CHAIN_LENGTH(6 downto 0) := DATA_IN(6 downto 0);
													end if;

			when WRITECHAIN_LENGTH_1 =>	if RX_DATA_RCVD = '1' then
														checksum <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
														DATA_OUT <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
														TX_SEND_DATA <= '1';
														DEBUG_STATE <= WRITECHAIN_ASSURED;
														CHAIN_LENGTH(13 downto 7) := DATA_IN(6 downto 0);
													end if;
													
			when WRITECHAIN_ASSURED =>	if RX_DATA_RCVD = '1' then
													if (DATA_IN = checksum) then
														DEBUG_STATE <= WRITECHAIN_DATA;
														checksum <= x"00";
													else
														DEBUG_STATE <= IDLE;
													end if;
												end if;
													
			when WRITECHAIN_DATA =>		if RX_DATA_RCVD = '1' then
														checksum <= checksum xor DATA_IN;
														SCAN_ENABLE <= '1';
														CHAIN_SINGLEBYTE := "111";
														CHAIN_DATA <= DATA_IN(6 downto 0);
														DEBUG_STATE <= WRITECHAIN_CLK0;
													end if;
													
			when WRITECHAIN_CLK1 =>		SCAN_ENABLE <= '1';
													CHAIN_WRITE_ENABLE <= '1';
													if (CHAIN_LENGTH > "00000000000000") AND (CHAIN_SINGLEBYTE > "000") then
														CHAIN_LENGTH := CHAIN_LENGTH - "00000000000001";
														CHAIN_SINGLEBYTE := CHAIN_SINGLEBYTE - "001";
														--NOSCAN_CLK(3 downto 0) <= "0000";
														case (CHAIN_ID) is 
															when "11" =>	CHAIN_WR_OUT(3) <= CHAIN_DATA(0);
																				SCAN_CLK <= "1000";
															when "10" =>	CHAIN_WR_OUT(2) <= CHAIN_DATA(0);
																				SCAN_CLK <= "0100";
															when "01" =>	CHAIN_WR_OUT(1) <= CHAIN_DATA(0);
																				SCAN_CLK <= "0010";
															when others =>	CHAIN_WR_OUT(0) <= CHAIN_DATA(0);
																				SCAN_CLK <= "0001";
														end case;
														CHAIN_DATA <= CHAIN_DATA(0) & CHAIN_DATA(6 downto 1);
														DEBUG_STATE <= WRITECHAIN_CLK0;
													elsif (CHAIN_LENGTH > "00000000000000") AND (CHAIN_SINGLEBYTE = "000") then
														DEBUG_STATE <= WRITECHAIN_DATA; -- byte done, wait for another
													elsif CHAIN_LENGTH = "00000000000000" then
														if TX_BUSY = '1' then
															DEBUG_STATE <= WRITECHAIN_WAIT_RS232;
														else
															DATA_OUT <= '1' & checksum(6 downto 0);
															TX_SEND_DATA <= '1';
															DEBUG_STATE <= IDLE;
														end if;
													end if;
													
			when WRITECHAIN_CLK0 =>		SCAN_ENABLE <= '1';
													CHAIN_WRITE_ENABLE <= '1';
													case (CHAIN_ID) is 
														when "11" => CHAIN_WR_OUT(3)<= CHAIN_DATA(0);
														when "10" => CHAIN_WR_OUT(2)<= CHAIN_DATA(0);
														when "01" => CHAIN_WR_OUT(1)<= CHAIN_DATA(0);
														when others => CHAIN_WR_OUT(0)<= CHAIN_DATA(0);
													end case;
													DEBUG_STATE <= WRITECHAIN_CLK1;
													
			when WRITECHAIN_WAIT_RS232	=> if TX_BUSY = '0' then
														DATA_OUT <= '1' & checksum(6 downto 0);
														TX_SEND_DATA <= '1';
														DEBUG_STATE <= IDLE;
													end if;
			
			
		----- READCHAIN
			when READCHAIN_LENGTH_0 =>	if RX_DATA_RCVD = '1' then
														checksum <= checksum xor DATA_IN;
														CHAIN_LENGTH(6 downto 0) := DATA_IN(6 downto 0);
														DEBUG_STATE <= READCHAIN_LENGTH_1;
													end if;
													
			when READCHAIN_LENGTH_1 =>	if RX_DATA_RCVD = '1' then
														checksum <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
														DATA_OUT <= '1' & (checksum(6 downto 0) xor DATA_IN(6 downto 0));
														TX_SEND_DATA <= '1';
														CHAIN_LENGTH(13 downto 7) := DATA_IN(6 downto 0);
														DEBUG_STATE <= READCHAIN_ASSURED;
													end if;
			
			when READCHAIN_ASSURED =>		if RX_DATA_RCVD = '1' then
														if (DATA_IN = checksum) then
															checksum <= x"00";
															CHAIN_SINGLEBYTE := "111";
															DEBUG_STATE <= READCHAIN_CLK0;
														else
															DEBUG_STATE <= IDLE;
														end if;
													end if;

			when READCHAIN_CLK0 =>			
													if (CHAIN_SINGLEBYTE = "000") then
													-- byte is full, send it
														CHAIN_SINGLEBYTE := "111";
														DEBUG_STATE <= READCHAIN_SENDBITS;
													else 
														CHAIN_SINGLEBYTE := CHAIN_SINGLEBYTE - "001";
														
														if (CHAIN_LENGTH = "00000000000000") then
															-- no more data to send, just pad with '0's
															CHAIN_DATA <= '0' & CHAIN_DATA(6 downto 1);
															DEBUG_STATE <= READCHAIN_CLK0;
														else
															-- some data left to send: get them and pack it into chain_data
															CHAIN_LENGTH := CHAIN_LENGTH - "00000000000001";

															SCAN_ENABLE <= '1';
															case (CHAIN_ID) is 
																when "11" => CHAIN_DATA <= CHAIN_IN(3) & CHAIN_DATA(6 downto 1);
																when "10" => CHAIN_DATA <= CHAIN_IN(2) & CHAIN_DATA(6 downto 1);
																when "01" => CHAIN_DATA <= CHAIN_IN(1) & CHAIN_DATA(6 downto 1);
																when "00" => CHAIN_DATA <= CHAIN_IN(0) & CHAIN_DATA(6 downto 1);
																when others => NULL;
															end case;
															DEBUG_STATE <= READCHAIN_CLK1;
														end if;
														
													end if;
													
			when READCHAIN_CLK1 =>			SCAN_ENABLE <= '1';
													case (CHAIN_ID) is 
														when "11" => SCAN_CLK <= "1000";
														when "10" => SCAN_CLK <= "0100";
														when "01" => SCAN_CLK <= "0010";
														when "00" => SCAN_CLK <= "0001";
														when others => SCAN_CLK <= "0000";
													end case;
													DEBUG_STATE <= READCHAIN_CLK0;
													
			when READCHAIN_SENDBITS =>	if (TX_BUSY = '0') then
														checksum <= checksum xor ('1' & CHAIN_DATA);
														DATA_OUT <= '1' & CHAIN_DATA;
														TX_SEND_DATA <= '1';
														if (CHAIN_LENGTH = "00000000000000") then
															-- all done, finish
															DEBUG_STATE <= READCHAIN_CHECKSUM;
														else 
															-- some data left in chain, make new byte
															DEBUG_STATE <= READCHAIN_CLK0;
														end if;
													end if;
													
			when READCHAIN_CHECKSUM =>	if (TX_BUSY = '0') then
														DATA_OUT <= '1' & checksum(6 downto 0);
														TX_SEND_DATA <= '1';
														DEBUG_STATE <= IDLE;
													end if;

			when others =>					DEBUG_STATE <= IDLE;
		end case;
	end if;
end process P_STATE_CHG;



end RTL;
