------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      SCOTCH_SYSTEM
--
-- PURPOSE:     Topmodule of the whole Hardware without IO
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-28
-- LAST CHANGE: 2010-02-24 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.cfg.all;

entity scotch_system is
	port(
		--I/O Schnittstelle
		IO_addressed : in std_logic_vector (0 to slaves-1);
		IO_ready : in std_logic_vector (0 to slaves-1);
		IO_rd : out std_logic_vector (0 to slaves-1);
		IO_wr : out std_logic_vector (0 to slaves-1);
		IO_data : inout std_logic_vector (31 downto 0);
		IO_address : out std_logic_vector (31 downto 0);
		IO_clk : out std_logic;
		--Prozessor debug Leitungen
		dbg_a : out std_logic_vector (31 downto 0); 
		dbg_x : out std_logic_vector (31 downto 0); 
		dbg_y : out std_logic_vector (31 downto 0); 
		dbg_ir : out std_logic_vector (31 downto 0); 
		dbg_ic : out std_logic_vector (31 downto 0);
		-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh : out std_logic_vector (3 downto 0);  
		-- Internal Chip Debugging Connectors
		clk_hicovec_in : in  std_logic;
		clk_io_in : in  std_logic;
		clk_shell_in : in  std_logic;
		se_shell : in  std_logic;
		sdi_shell : in  std_logic;
		sdo_shell : out  std_logic
  );
end scotch_system;

architecture RTL of scotch_system is
-- HiCoVec
component hicovec is
	port(
		clk_hicovec: 	in std_logic;
		reset: 		in std_logic;       
		dbg_a: 		out std_logic_vector(31 downto 0);
		dbg_x:	 	out std_logic_vector(31 downto 0);
		dbg_y: 		out std_logic_vector(31 downto 0);
		dbg_ir: 	out std_logic_vector(31 downto 0);
		dbg_ic: 	out std_logic_vector(31 downto 0);
		dbg_carry: 	out std_logic;
		dbg_zero: 	out std_logic;
		dbg_ir_ready: 	out std_logic;
		dbg_halted: 	out std_logic;
		ready:		in std_logic;
		wr:             out std_logic;
		rd:             out std_logic;
		address:        out std_logic_vector(31 downto 0);
		data:		inout  std_logic_vector(31 downto 0)
    );
end component;
-- Debug_Shell		
component dbgshell is
	port( 
		clk, 
		se, sdi: in std_logic;
		sdo: out std_logic;
		cpu_reset: out std_logic;
		cpu_rd, cpu_wr: in std_logic;
		cpu_ready: out std_logic;	
		cpu_adr: in std_logic_vector (31 downto 0);
		cpu_data: inout std_logic_vector (31 downto 0);
		bus_rd, bus_wr: out std_logic;
		bus_ready: in std_logic;	
		bus_adr: out std_logic_vector (31 downto 0);
		bus_data: inout std_logic_vector (31 downto 0)
	);
end component;
-- Buslogik
component buslogic is
	generic (slaves: integer);
	port (
		ma_rd, ma_wr: in std_logic;
		ma_ready: out std_logic;
		sl_adressed, sl_ready: in std_logic_vector (0 to slaves-1);
		sl_rd, sl_wr: out std_logic_vector (0 to slaves-1)
	);
end component;


for all: hicovec use entity WORK.hicovec(RTL);
for all: dbgshell use entity WORK.dbgshell(RTL);
for all: buslogic use entity WORK.buslogic(RTL);

signal s_cpu_reset: std_logic;
signal s_cpu_ready: std_logic;
signal s_cpu_wr: std_logic;
signal s_cpu_rd: std_logic;
signal s_cpu_adr: std_logic_vector(31 downto 0);
signal s_cpu_data: std_logic_vector(31 downto 0);

signal s_clk_shell: std_logic;
signal s_se_shell: std_logic;
signal s_sdi_shell: std_logic;
signal s_sdo_shell: std_logic;

signal s_bus_rd: std_logic;
signal s_bus_wr: std_logic;
signal s_bus_ready: std_logic;

signal s_noscan_clk: std_logic_vector(3 downto 0);
signal s_scan_clk: std_logic_vector(3 downto 0);
signal s_chain_out: std_logic_vector(3 downto 0);
signal s_chain_in: std_logic_vector(3 downto 0);
signal s_scan_enable: std_logic;

signal s_dbg_czrh: std_logic_vector(3 downto 0);

begin
	hicovec_impl: hicovec
	port map(
		clk_hicovec	=> clk_hicovec_in,
		reset		=> s_cpu_reset,
		dbg_a		=> dbg_a,
		dbg_x		=> dbg_x,
		dbg_y		=> dbg_y,
		dbg_ir		=> dbg_ir,
		dbg_ic		=> dbg_ic,
		dbg_carry	=> s_dbg_czrh(0),
		dbg_zero	=> s_dbg_czrh(1),
		dbg_ir_ready	=> s_dbg_czrh(2),
		dbg_halted	=> s_dbg_czrh(3),
		ready		=> s_cpu_ready,
		wr		=> s_cpu_wr,
		rd		=> s_cpu_rd,
		address		=> s_cpu_adr,
		data		=> s_cpu_data
	);
	
	dbgshell_impl: dbgshell
	port map( 
		clk 		=> clk_shell_in,
		se		=> se_shell,
		sdi		=> sdi_shell,			
		sdo		=> sdo_shell,
		cpu_reset	=> s_cpu_reset,
		cpu_rd		=> s_cpu_rd,
		cpu_wr		=> s_cpu_wr,
		cpu_ready	=> s_cpu_ready,
		cpu_adr		=> s_cpu_adr,
		cpu_data        => s_cpu_data,
		bus_rd 		=> s_bus_rd,
		bus_wr		=> s_bus_wr,
		bus_ready	=> s_bus_ready,
		bus_adr		=> IO_address,
		bus_data	=> IO_data
	);

	buslogic_impl: buslogic generic map (slaves => slaves)
	port map(
		ma_rd		=> s_bus_rd,
		ma_wr		=> s_bus_wr,
		ma_ready	=> s_bus_ready,
		sl_adressed	=> IO_addressed, 
		sl_ready	=> IO_ready,
		sl_rd		=> IO_rd,
		sl_wr		=> IO_wr
	);
-- Glue Logic
	IO_clk			<= clk_io_in;
	dbg_czrh		<=  s_dbg_czrh;


end RTL;

