------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      SCOTCH
--
-- PURPOSE:     Topmodule of the whole Hardware
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
--
-- DATE CREATE: 2009-01-02
-- LAST CHANGE: 2010-01-10 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.cfg.all;

entity scotch is
	port(
		-- CLK und RS232
		CLK_50M : in  STD_LOGIC;
		RS232_DCE_RXD : in  STD_LOGIC;
		RS232_DCE_TXD : out STD_LOGIC;
		-- I/O MISC
		LED : out std_logic_vector(7 downto 0);
		BTN_EAST, BTN_NORTH, BTN_SOUTH, BTN_WEST: in std_logic;
		ROT_CENTER, ROT_A, ROT_B: in std_logic;
		SW: in std_logic_vector(0 to 3);
		J18_IO: in std_logic_vector(0 to 3);
		J20_IO: in std_logic_vector(0 to 3);
		-- LCD Display
		LCD_DB : out std_logic_vector(7 downto 0);
		LCD_E : out std_logic;
		LCD_RS : out std_logic;
		LCD_RW : out std_logic;
		-- VGA
		VGA_R   : out std_logic_vector (3 downto 0);
		VGA_G 	: out std_logic_vector (3 downto 0);
		VGA_B  	: out std_logic_vector (3 downto 0);
		VGA_HSYNC    : out std_logic;
		VGA_VSYNC    : out std_logic;
		--Audio
		AUDIO_L : out std_logic;
		AUDIO_R : out std_logic
	);
end scotch;

architecture RTL of scotch is
-- Scotch System
component scotch_system is
	port(
	--I/O Schnittstelle
		IO_addressed : in std_logic_vector (0 to slaves-1);
		IO_ready : in std_logic_vector (0 to slaves-1);
		IO_rd : out std_logic_vector (0 to slaves-1);
		IO_wr : out std_logic_vector (0 to slaves-1);
		IO_data : inout std_logic_vector (31 downto 0);
		IO_address : out std_logic_vector (31 downto 0);
		IO_clk : out std_logic;
	--Prozessor debug Leitungen
		dbg_a : out std_logic_vector (31 downto 0); 
		dbg_x : out std_logic_vector (31 downto 0); 
		dbg_y : out std_logic_vector (31 downto 0); 
		dbg_ir : out std_logic_vector (31 downto 0); 
		dbg_ic : out std_logic_vector (31 downto 0);
	-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh : out std_logic_vector (3 downto 0);
	-- Signale zum Debugger
		clk_hicovec_in : in  std_logic;
		clk_io_in : in  std_logic;
		clk_shell_in : in  std_logic;
		se_shell : in  std_logic;
		sdi_shell : in  std_logic;
		sdo_shell : out  std_logic  
	);
end component;
-- Debugger
component debugger is
	port(
		-- Clock and SerialPort
		CLK_50MHZ : in  STD_LOGIC;
		RS232_RXD : in  STD_LOGIC;
		RS232_TXD : out STD_LOGIC;		
		-- Internal Chip Debugging Connectors
		-- clocks off scan path
		NOSCAN_CLK  : out STD_LOGIC_VECTOR(3 downto 0);
		-- clocks on scan path
		SCAN_CLK    : out STD_LOGIC_VECTOR(3 downto 0);
		SCAN_ENABLE : out STD_LOGIC;
		-- data to/from chain
		CHAIN_OUT   : out STD_LOGIC_VECTOR(3 downto 0);
		CHAIN_IN    : in  STD_LOGIC_VECTOR(3 downto 0)
	);
end component;
-- Blockram
component blockram is
	port (
		clk: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;      
		rd, wr: in std_logic;
		ready: out std_logic
	);
end component;
-- VGA
component vga is
	port (
		clk: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;      
		rd, wr: in std_logic;
		ready: out std_logic;
		-- VGA_IO
		CLK_50M  : in std_logic;
		VGA_R   : out std_logic_vector (3 downto 0);
		VGA_G 	: out std_logic_vector (3 downto 0);
		VGA_B  	: out std_logic_vector (3 downto 0);
		VGA_HSYNC    : out std_logic;
		VGA_VSYNC    : out std_logic;
		--Eingänge von ASCII
		do_ascii: in std_logic;
		ascii_data: inout std_logic_vector (31 downto 0);
		ascii_adr_in: in std_logic_vector (31 downto 0);
		ascii_wr: inout std_logic;
		ascii_ready: out std_logic	
	);
end component;
--Debug IO
component debug_io is
	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		dbg_a: in std_logic_vector (31 downto 0); 
		dbg_x: in std_logic_vector (31 downto 0); 
		dbg_y: in std_logic_vector (31 downto 0); 
		dbg_ir: in std_logic_vector (31 downto 0); 
		dbg_ic: in std_logic_vector (31 downto 0);
	-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh: in std_logic_vector (3 downto 0)  
	);
end component;
-- IO MISC
component iomisc is
	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		LED : out std_logic_vector(7 downto 0);
		BTN_EAST, BTN_NORTH, BTN_SOUTH, BTN_WEST: in std_logic;
		ROT_CENTER, ROT_A, ROT_B: in std_logic;
		SW: in std_logic_vector(0 to 3);
		J18_IO: in std_logic_vector(0 to 3);
		J20_IO: in std_logic_vector(0 to 3)
	);
end component;
--LCD
component lcd is
	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		LCD_DB : out std_logic_vector(7 downto 0);
		LCD_E : out std_logic;
		LCD_RS : out std_logic;
		LCD_RW : out std_logic
	);
end component;
-- ASCII
component ascii is
	port(
		-- BUS Verbindung
		clk: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- Verbindung zum VGA RAM
		do_ascii: out std_logic;
		vga_data: inout std_logic_vector (31 downto 0);
		vga_adr: out std_logic_vector (31 downto 0);
		vga_wr: out std_logic;
		vga_ready: in std_logic
	);
end component;

--Audio

component audio is
	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		AUDIO_L: out std_logic;
		AUDIO_R: out std_logic
	);
end component;


for all: scotch_system use entity WORK.scotch_system(RTL);
for all: debugger use entity WORK.debugger(RTL);
for all: blockram use entity WORK.blockram(RTL);
for all: vga use entity WORK.vga(RTL);
for all: debug_io use entity WORK.debug_io(RTL);
for all: iomisc use entity WORK.iomisc(RTL);
for all: lcd use entity WORK.lcd(RTL);
for all: ascii use entity WORK.ascii(RTL);
for all: audio use entity WORK.audio(RTL);
-- System --> IO
signal s_CLK_50M: std_logic;
signal s_IO_adressed: std_logic_vector(0 to slaves-1);
signal s_IO_ready: std_logic_vector(0 to slaves-1);
signal s_IO_rd: std_logic_vector(0 to slaves-1);
signal s_IO_wr: std_logic_vector(0 to slaves-1);
signal s_IO_data: std_logic_vector(31 downto 0);
signal s_IO_adress: std_logic_vector(31 downto 0);
signal s_IO_clk: std_logic;
-- System --> Debug_IO
signal s_dbg_a: std_logic_vector (31 downto 0);
signal s_dbg_x: std_logic_vector (31 downto 0);
signal s_dbg_y: std_logic_vector (31 downto 0);
signal s_dbg_ir: std_logic_vector (31 downto 0);
signal s_dbg_ic: std_logic_vector (31 downto 0);
signal s_dbg_czrh: std_logic_vector (3 downto 0);
-- Signale Debugger
signal s_noscan_clk: std_logic_vector(3 downto 0);
signal s_scan_clk: std_logic_vector(3 downto 0);
signal s_chain_out: std_logic_vector(3 downto 0);
signal s_chain_in: std_logic_vector(3 downto 0);
signal s_scan_enable: std_logic;
signal s_clk_hicovec_in: std_logic;
signal s_clk_io_in: std_logic;
signal s_clk_shell_in: std_logic;
signal s_se_shell: std_logic;
signal s_sdi_shell: std_logic;
signal s_sdo_shell: std_logic;
--Signale ASCII
signal s_do_ascii: std_logic;
signal s_ascii_data_in: std_logic_vector (31 downto 0);
signal s_ascii_adr_in: std_logic_vector (31 downto 0);
signal s_ascii_wr: std_logic;
signal s_ascii_ready: std_logic;

begin
	scotch_system_impl: scotch_system
	port map (
	--I/O Schnittstelle
		IO_addressed	=> s_IO_adressed,
		IO_ready	=> s_IO_ready,
		IO_rd		=> s_IO_rd,
		IO_wr		=> s_IO_wr,
		IO_data		=> s_IO_data,
		IO_address	=> s_IO_adress,
		IO_clk		=> s_IO_clk,
	--Prozessor debug Leitungen
		dbg_a		=> s_dbg_a,
		dbg_x		=> s_dbg_x,
		dbg_y		=> s_dbg_y,
		dbg_ir		=> s_dbg_ir,
		dbg_ic		=> s_dbg_ic,
	-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh	=> s_dbg_czrh,
	-- Signale zum Debugger
		clk_hicovec_in 	=> s_clk_hicovec_in,
		clk_io_in	=> s_clk_io_in,
		clk_shell_in 	=> s_clk_shell_in,
		se_shell	=> s_se_shell,
		sdi_shell	=> s_sdi_shell,
		sdo_shell 	=> s_sdo_shell
  );
	debugger_impl: debugger
	port map(
		CLK_50MHZ	=> s_CLK_50M,
		RS232_RXD	=> RS232_DCE_RXD,
		RS232_TXD	=> RS232_DCE_TXD,
		NOSCAN_CLK	=> s_noscan_clk,
		SCAN_CLK	=> s_scan_clk,
		SCAN_ENABLE	=> s_scan_enable,
		CHAIN_OUT	=> s_chain_out,
		CHAIN_IN	=> s_chain_in
      );
	blockram_impl: blockram
	port map(
		clk		=> s_IO_clk,
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(0),
		rd		=> s_IO_rd(0),
		wr		=> s_IO_wr(0),
		ready		=> s_IO_ready(0)
	);
	vga_impl: vga
	port map(
		clk		=> s_IO_clk,
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(1),
		rd		=> s_IO_rd(1),
		wr		=> s_IO_wr(1),
		ready		=> s_IO_ready(1),
		-- VGA_IO
		CLK_50M  	=> s_CLK_50M,
		VGA_R  		=> VGA_R,
		VGA_G  		=> VGA_G,
		VGA_B  		=> VGA_B,
		VGA_HSYNC  	=> VGA_HSYNC,
		VGA_VSYNC  	=> VGA_VSYNC,
			--Eingänge von ASCII
		do_ascii	=> s_do_ascii,
		ascii_data  	=> s_ascii_data_in,
		ascii_adr_in  	=> s_ascii_adr_in,
		ascii_wr  	=> s_ascii_wr,
		ascii_ready  	=> s_ascii_ready
	);
	debug_io_impl: debug_io
	port map(
		clk		=> s_IO_clk,
		-- bus interface...
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(2),
		rd		=> s_IO_rd(2),
		wr		=> s_IO_wr(2),
		ready		=> s_IO_ready(2),
		-- I/O ports...
		dbg_a		=> s_dbg_a,
		dbg_x		=> s_dbg_x,
		dbg_y		=> s_dbg_y,
		dbg_ir		=> s_dbg_ir,
		dbg_ic		=> s_dbg_ic,
	-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh	=> s_dbg_czrh
	);
	iomisc_impl: iomisc
	port map (
		clk		=> s_IO_clk,
		-- bus interface...
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(3),
		rd		=> s_IO_rd(3),
		wr		=> s_IO_wr(3),
		ready		=> s_IO_ready(3),
		-- I/O ports...
		LED 		=> LED,
		BTN_EAST	=> BTN_EAST,
		BTN_NORTH	=> BTN_NORTH,
		BTN_SOUTH	=> BTN_SOUTH,
		BTN_WEST	=> BTN_WEST,
		ROT_CENTER	=> ROT_CENTER,
		ROT_A		=> ROT_A,
		ROT_B		=> ROT_B,
		SW		=> SW,
		J18_IO => J18_IO,
		J20_IO => J20_IO
	);
	lcd_impl : lcd
	port map (
		clk		=> s_IO_clk,
		-- bus interface...
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(4),
		rd		=> s_IO_rd(4),
		wr		=> s_IO_wr(4),
		ready		=> s_IO_ready(4),
      -- I/O ports...
		LCD_DB		=> LCD_DB,
		LCD_E		=> LCD_E,
		LCD_RS		=> LCD_RS,
		LCD_RW 		=> LCD_RW
	);
	ascii_impl : ascii
	port map(
	-- BUS Verbindung
		clk		=> s_IO_clk,
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(5),
		rd		=> s_IO_rd(5),
		wr		=> s_IO_wr(5),
		ready		=> s_IO_ready(5),
	-- Verbindung zum VGA RAM
		do_ascii	=> s_do_ascii,
		vga_data	=> s_ascii_data_in,
		vga_adr		=> s_ascii_adr_in,
		vga_wr		=> s_ascii_wr,
		vga_ready	=> s_ascii_ready
	);
	audio_impl : audio
	port map (
		clk		=> s_IO_clk,
		-- bus interface...
		adr		=> s_IO_adress,
		data		=> s_IO_data,
		adressed	=> s_IO_adressed(6),
		rd		=> s_IO_rd(6),
		wr		=> s_IO_wr(6),
		ready		=> s_IO_ready(6),
      -- I/O ports...
		AUDIO_L		=> AUDIO_L,
		AUDIO_R		=> AUDIO_R

	);

	
	-- Glue Logic
	s_CLK_50M		<= CLK_50M;
	s_clk_hicovec_in	<= s_noscan_clk(1);
	s_clk_io_in		<= s_noscan_clk(2);
	s_clk_shell_in		<= s_scan_clk(0);
	s_se_shell		<= s_scan_enable;
	s_sdi_shell		<= s_chain_out(0);
	s_chain_in		<= "000" & s_sdo_shell;
end RTL;

