/*
*  Filename: chipdebuglib.c
*  Author: Michael Schaeferling
*  Institute: Hochschule Augsburg, Fakultaet fuer Informatik
*  Date of last change: 09-03-23
*  About: This file is part of the OnChipDebugger (via RS232).
*         This library needs to be used when interfacing the serial debugging hardware.
*  Version: 0.51
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <pthread.h> /* POSIX threads */
#include <signal.h>
#include <sys/time.h>



/*********************************************
 *  Instructions from SW-Lib to HW-Debugger  *
 *********************************************/
#define I_STEP		0x01
#define I_RUN		0x0D
#define I_STOP		0x05

#define I_WRITECHAIN	0x0E
#define I_READCHAIN	0x06

/* for later use
#define I_SETPIN	0x0A
#define I_GETPIN	0x02
*/

/*********************************************
 *  Acknowledges from HW-Debugger to SW-Lib  *
 *********************************************/
#define ACK_ALIVE	0x41	/* "I'm Alive" word from HW */
#define ACK_DONE	0x4F	/* "I'm Done" word from HW */
 
/* number of bytes to be read from the serial port at one access */
#define MAXREAD 512

/* the ringbuffers count of bytes */
#define RINGBUFSIZE 1024

/* how long (in seconds) may the rx-thread take to get up so that the main thread can work with data? */
#define RXREADY_TIMEOUT 5

/* how long (in seconds) may the hardware need to send an instructions result back to the sw-lib?
 this is used in InternalRead for timing out the read access */
#define MAXREADDELAY 3

/* how long (in seconds) may the hardware take to send an alive word? when exceeding this time the hardware is supposed to be offline*/
#define MAXTIMESTAMPDELTA 3


 
/* alive-info */
typedef struct {
  int info;
  time_t timestamp;
} s_alive;

volatile s_alive g_alive;

/* rx-ringbuffer for fetched data, protected by mutex */
typedef struct {
  int datacount;
  char data[RINGBUFSIZE];
  int writepos;
  int readpos;
} s_data_rx;

pthread_mutex_t data_rx_mtx;
volatile s_data_rx g_data_rx;

pthread_cond_t sig_data_rx, sig_buf_free = PTHREAD_COND_INITIALIZER;

volatile int fd = -1;
volatile int verbosity = 1;  /* 0 - no output; 1 - normal output; 2 - full debug output */

volatile int kill_rx = 0;
volatile int rx_ready = 0;

pthread_t rx_thread;


int FlushInputBuffer(){
  char delete[128];
  int n, flushed = 0;
  /*while(read(fd, delete, 1) == -1);*/
  while((n = read(fd, delete, 128)) > 1) flushed += n;
  
  if (verbosity > 0) printf("FlushInputBuffer() wiped %i bytes.\n", flushed);
  
  return flushed;
}


/* for rx data process */
int ParseResponse(int count, char *response){
  int i, locked = 0;
  
  g_alive.info = 0;

  pthread_mutex_lock(&data_rx_mtx);
  locked = 1;

  for (i = 0; i < count; i++){
    if (response[i] == ACK_ALIVE){
      pthread_mutex_unlock(&data_rx_mtx);
      locked = 0;
        /* set timestamp according to alive-data */
        g_alive.info = 1;
        g_alive.timestamp = time(NULL);
      if (i < (count - 1)){
        pthread_mutex_lock(&data_rx_mtx);
        locked = 1;
      }
    }

    else{
      /* get non-alive data and put them into the ringbuffer if there's some space left - otherwise wait until there is */
      while(g_data_rx.datacount == RINGBUFSIZE){
        /* if this should ever happen, just use the following line and the pthread_cond_signal command in InternalRead() to avoid busy waiting
        pthread_cond_wait(&sig_buf_free, &data_rx_mtx); */
        if (verbosity > 0) fputs("Ringbuffer is full...RX-thread now waits, so some data may get lost!\n", stderr);
      }

      g_data_rx.datacount++;
      g_data_rx.data[g_data_rx.writepos] = response[i];
      
      if (verbosity > 1) printf("%i : 0x%.2x \n", g_data_rx.writepos, (unsigned char)g_data_rx.data[g_data_rx.writepos]);
      (g_data_rx.writepos < RINGBUFSIZE-1) ? (g_data_rx.writepos++) : (g_data_rx.writepos = 0);

    }
  }

  if (locked == 1){
    pthread_mutex_unlock(&data_rx_mtx);
    pthread_cond_signal(&sig_data_rx);
  }

  return 0;
}

/* main function for rx-thread */
void* rxdata(){
  char data[MAXREAD + 1];
  int nbytes;
  
  /* Some serial drivers don't flush their buffers although opening the device with the F_SETFL flag. So we may do it manually:
  FlushInputBuffer(); */
  
  rx_ready = 1;
  
  while(kill_rx == 0){
    nbytes = read(fd, data, MAXREAD); /* blocking here if there's nothing to fetch */
    if (nbytes > 0){
      data[nbytes] = '\0';
      if (strlen(data) == nbytes){
        ParseResponse(nbytes, data);
        /* if (verbosity > 1) printf("Read done (%i). Alive:%i \n", nbytes, g_alive.info); */
      }
      else{ 
        if (verbosity > 0) fputs("Error reading port: strlen(data) != nbytes. Possibly 0x0 was sent from debugger.\n", stderr);
      }
    }
    else if (nbytes < 0){ 
      if (verbosity > 0) fputs("Error reading port: Result was negative.\n", stderr);
    }
  }

  rx_ready = 0;
  kill_rx = 0;

  return NULL;
}


static int open_port(char* c_port){
  int fd; /* File descriptor for the port */
  struct termios options;
	
  fd = open(c_port, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1){
    /* Could not open the port. */
    if (verbosity > 0){
      char* errmsg0 = "open_port(): Unable to open port ";
      char* errmsg = (char*)malloc((strlen(errmsg0) + strlen(c_port) + 1)*sizeof(char));
      strcpy(errmsg, errmsg0);
      strcat(errmsg, c_port);
      perror(errmsg);
      free(errmsg);
    }
    return -1;
  }
  else{
    fcntl(fd, F_SETFL, 0);
    tcgetattr(fd, &options);
      options.c_cflag = B38400 | CS8 | CLOCAL | CREAD;
      options.c_iflag = IGNPAR;
      options.c_oflag = 0;
      options.c_lflag = 0;
      options.c_cc[VMIN]  = 0; /* at least one byte needs to fetched; this causes a read to block if there's nothing received */
      options.c_cc[VTIME] = 30; /* in 1/10sec => 30 = 3 sec */
    tcsetattr(fd, TCSANOW, &options);
    return (fd); 
  }
}


int InternalWrite(int count, char *dataBytes){
  int n = write(fd, dataBytes, count);
  if (n != count){
    if (verbosity > 0) fputs("Write() failed! Not all bytes were sent. Sorry.\n", stderr);
    return -1;
  }
  return 0;
}

int InternalRead(int count, char *dataBytes){
  int i = 0;
  int rc;
  struct timespec ts;
  struct timeval tp;

  while(i < count){
      
    pthread_mutex_lock(&data_rx_mtx);
    /* get as many bytes from buffer as specified; this will fetch a whole data-chunk */
    while ((g_data_rx.datacount != 0) && (i < count)){
      g_data_rx.datacount--;
      dataBytes[i] = g_data_rx.data[g_data_rx.readpos];
      i++;
      (g_data_rx.readpos < RINGBUFSIZE-1) ? (g_data_rx.readpos++) : (g_data_rx.readpos = 0);
    }

    if (i < count){
      gettimeofday(&tp, NULL);
      ts.tv_sec  = tp.tv_sec;
      ts.tv_nsec = tp.tv_usec * 1000;
      ts.tv_sec += MAXREADDELAY;
      rc = pthread_cond_timedwait(&sig_data_rx, &data_rx_mtx, &ts); /* wait for some more data to come into the buffer */
      if (rc == ETIMEDOUT){
        pthread_mutex_unlock(&data_rx_mtx);
        if (verbosity > 0) fputs("InternalRead(): Read operation on ringbuffer timed out.\n", stderr);
        if (verbosity > 1) printf("Possible reason: missing %i dataword(s).\n", count - i);
        return -1;
      }
    }
    pthread_mutex_unlock(&data_rx_mtx);
    
    /* pthread_cond_signal(&sig_buf_free); */
  }
  return i;
}


char Checksum(char* dataBytes, int size){
  char checksum = 0;
  int i;
  for(i = 0; i < size; i++){
    checksum ^= (unsigned char)dataBytes[i];
  }
  return checksum;
}



int SetVerbosity(int verb){
  if ((verbosity<0)||(verbosity>2)){
    fputs("SetVerbosity failed. Select value between 0 (no output) and 2 (full debug output)", stderr);
    return -1;
  }
  else verbosity = verb;
  return 0;
}

int GetVerbosity(){
  return verbosity;
}


int Open (char *c_port) {

  pthread_cond_init(&sig_data_rx, NULL);
  pthread_mutex_init(&data_rx_mtx, NULL);
  g_alive.info = 0;

  fd = open_port(c_port);
  if (fd < 0) return -1;
  else{
    g_data_rx.writepos = 0;
    g_data_rx.readpos = 0;

    if (pthread_create (&rx_thread, NULL, rxdata, NULL) != 0) {
      puts ("Error creating rx-thread. \n");
      exit (3);
    }

    time_t reftime = time(NULL);
    while (rx_ready != 1){
      if (time(NULL) - reftime > RXREADY_TIMEOUT){
        if (verbosity > 0) fputs("Open() failed! Getting the rx-thread up timed out.\n", stderr);
        close (fd);
        return -1;
      }
    }
  }
  return 0;
}


void Close () {
  if (fd >= 0){
    kill_rx = 1;
    pthread_join(rx_thread, NULL);
    close (fd);
  }

  g_alive.info = 0;
  pthread_cond_destroy(&sig_data_rx);
  pthread_mutex_destroy(&data_rx_mtx);
}


int IsAlive(){
  int alive = 0;
  if ((g_alive.info == 1) && ((time(NULL) - g_alive.timestamp) < MAXTIMESTAMPDELTA)){
    if (verbosity > 1) printf("Device is alive. time elapsed since last \"Hello\": %i seconds.\n", (int)(time(NULL) - g_alive.timestamp));
    alive = 1;
  }
  return alive;
}


int WriteChain(int chainID, char* c_to_chip){
  int chain_len = strlen(c_to_chip);
  int dataByteCount;
  int i, j;
  char checksum, hwchecksum, bytechecksum;
  char instructionBytes[4];
  
  if (chain_len == 0){
    if (verbosity > 0) fputs("WriteChain() failed! Chain has at least one bit.\n", stderr);
    return -1;
  }
  if (chain_len > 16383){
    if (verbosity > 0) fputs("WriteChain() failed! Chain too long, max 16383 bits.\n", stderr);
    return -1;
  }
  if ((chainID < 0) || (chainID > 3)){
    if (verbosity > 0) fputs("WriteChain() failed! Max 4 chains (0-3) supported.\n", stderr);
    return -1;
  }

  /* Bytes to send: */
  dataByteCount = (chain_len / 7) + ((chain_len%7 > 0) ? 1 : 0);
    /* malloc/free-style: char* dataBytes = (char*)calloc((dataByteCount + 1),sizeof(char)); */
  char dataBytes[dataByteCount + 1];
  
  /* generate instruction byte */
  instructionBytes[0] = (char)I_WRITECHAIN;
  instructionBytes[0] &= 0x9F;
  instructionBytes[0] |= (chainID << 5);

  /* generate size bytes */
  instructionBytes[1] = (char)((chain_len & 0x7F) | 0x80); /* use lower bits only */
  instructionBytes[2] = (char)((chain_len / 128)  | 0x80); /* use upper bits only */

  instructionBytes[3] = '\0';
  
  /* generate data bytes */
  for (i = 0; i < dataByteCount; i++){
    for (j = 0; j < 7; j++){
      switch(c_to_chip[(i*7)+j]){
        case '1':	dataBytes[i] |= 0x80;	/* set first bit */
			break;
        case '0':	dataBytes[i] &= 0x7F;	/* clear first bit */
			break;
        case '\0': /* fill our last byte to be compliant */
			dataBytes[i] = (char)((unsigned char)dataBytes[i] >> (6 - j));
			j = 7;
			break;
        default:	if (verbosity > 0) fputs("WriteChain() failed! Invalid character in chain\n", stderr);
			break;
      }
      dataBytes[i] = (char)((unsigned char)dataBytes[i] >> 1);
    }
    dataBytes[i] |= 0x80; /* set first bit for a data packet */
  }

  dataBytes[dataByteCount] = '\0';
  

  checksum = (unsigned char)(Checksum(instructionBytes, strlen(instructionBytes)) | 0x80);
  bytechecksum = (unsigned char)(Checksum(dataBytes, strlen(dataBytes)) | 0x80);

/* send instruction */
  if (InternalWrite(strlen(instructionBytes), instructionBytes) != 0){
    if (verbosity > 0) fputs("WriteChain() failed in InternalWrite()!\n", stderr);
    return -1;
  }

  
/* the hardware proceeds its work when receiving the corresponding checksum */
  InternalWrite(1, &checksum);
  /* get response - this should be our checksum (for error handling) */
  if (InternalRead(1, &hwchecksum) == -1){
    if (verbosity > 0) fputs("InternalRead() for Instruction Checksum in WriteChain() failed.\n", stderr);
    return -1;
  }
  if (hwchecksum != checksum){
    if (verbosity > 0) fputs("WriteChain() failed! Instruction Checksum error.\n", stderr);
    printf("Wi: sw=0x%.2x ; hw=0x%.2x \n", checksum, hwchecksum);
    return -2;
  }


/* send data */
  if (InternalWrite(strlen(dataBytes), dataBytes) != 0){
    if (verbosity > 0) fputs("WriteChain() failed in InternalWrite()!\n", stderr);
    /* malloc/free-style: free (dataBytes); */
    return -1;
  }

/* get response - this should be our bytechecksum now */
  if (InternalRead(1, &hwchecksum) == -1){
    if (verbosity > 0) fputs("InternalRead() for Byte Checksum in WriteChain() failed.\n", stderr);
    return -1;
  }
  if (hwchecksum != bytechecksum){
    if (verbosity > 0) fputs("WriteChain() failed! Byte Checksum error.\n", stderr);
    printf("Wb: sw=0x%.2x ; hw=0x%.2x \n", checksum, hwchecksum);
    return -3;
  }

  if (verbosity > 1){
    printf("\nDEBUG (writechain): c_len=%d ; bc=%d ; str_len=%d ; n=%d ; db1=0x%.2x ; db2=0x%.2x ; cs=0x%.2x\n\n", chain_len, dataByteCount, strlen(dataBytes), dataByteCount + 3, dataBytes[1], dataBytes[2], checksum);
    for (i = 0; i < strlen(dataBytes); i++)
    printf("0x%.2x ", (unsigned char)dataBytes[i]);
    printf("\n\n");
  }

  /* malloc/free-style: free (dataBytes); */

  return 0;
}


int ReadChain(int chainID, char* c_from_chip){
  int i, j = 0;
  char checksum, hwchecksum, bytechecksum;
  char instructionBytes[4];
  
  int chain_len = strlen(c_from_chip);
  
  int dataByteCount = (chain_len / 7) + ((chain_len%7 > 0) ? 1 : 0);
  
  /* malloc/free-style: char* dataBytes = (char*)calloc(dataByteCount,sizeof(char)); */
  char dataBytes[dataByteCount];
  
  if (chain_len == 0){
    if (verbosity > 0) fputs("ReadChain() failed! Chain has at least one bit.\n", stderr);
    return -1;
  }
  if (chain_len > 16383){
    if (verbosity > 0) fputs("ReadChain() failed! Chain too long, max 16383 bits.\n", stderr);
    return -1;
  }
  if ((chainID < 0) || (chainID > 3)){
    if (verbosity > 0) fputs("WriteChain() failed! Max 4 chains (0-3) supported.\n", stderr);
    return -1;
  }

  /* generate instruction byte */
  instructionBytes[0] = (char)I_READCHAIN;
  instructionBytes[0] &= 0x9F;
  instructionBytes[0] |= (chainID << 5);

  /* generate size bytes */
  instructionBytes[1] = (char)((chain_len & 0x7F) | 0x80); /* use lower bits only */
  instructionBytes[2] = (char)((chain_len / 128)  | 0x80); /* use upper bits only */
  
  instructionBytes[3] = '\0';

  checksum = (Checksum(instructionBytes, strlen(instructionBytes)) | 0x80);

/* send instruction */
  InternalWrite(strlen(instructionBytes), instructionBytes);
  
  if (verbosity > 1){
    printf("\nDEBUG (readchain): c_len=%d ; bc=%d ; str_len=%d ; db1=0x%.2x ; db2=0x%.2x ; cs=0x%.2x \n\n", chain_len, dataByteCount, strlen(instructionBytes), instructionBytes[1], instructionBytes[2], checksum);
    for (i = 0; i < strlen(instructionBytes); i++)
      printf("0x%.2x ", (unsigned char)instructionBytes[i]);
    printf("\n\n");
  }

  /* the hardware proceeds its work when receiving the corresponding checksum */
  InternalWrite(1, &checksum);
  /* get response - this should be our checksum (for error handling) and the data-bytes */
  if (InternalRead(1, &hwchecksum) == -1){
    if (verbosity > 0) fputs("InternalRead() for Instruction Checksum in ReadChain() failed.\n", stderr);
    return -1;
  }
  if (hwchecksum != checksum){
    if (verbosity > 0) fputs("ReadChain() failed! Instruction checksum error.\n", stderr);
    printf("Ri: sw=0x%.2x ; hw=0x%.2x \n", checksum, hwchecksum);
    return -2;
  }
  
  /* get the data bytes. caution! this read may fail due to heavy load on the serial cable or scarce timeout definition in the read function. */
  if (InternalRead(dataByteCount, dataBytes) == -1){
    if (verbosity > 0) fputs("InternalRead() for DataBytes in ReadChain() failed.\n", stderr);
    return -1;
  }

  /* build checksum for received bytes */
  bytechecksum = (Checksum(dataBytes, dataByteCount) | 0x80);

  /* get checksum over all databytes. could be used to compare with received data */
  if (InternalRead(1, &hwchecksum) == -1){
	if (verbosity > 0) fputs("InternalRead() for Data Checksum in ReadChain() failed.\n", stderr);
    return -1;
  }

  if (verbosity > 1){
    printf("Got %d DataByte(s):\n", dataByteCount);
    for (i = 0; i < dataByteCount; i++)
      printf("0x%.2x ", (unsigned char)dataBytes[i]);
    printf("\n");
  }

  /* build a chain of 1's and 0's to be returned */
  for (i = 0; i < dataByteCount; i++){
    if (((unsigned char)dataBytes[i] & 0x80) == 0x80){
      if (verbosity > 1) printf("Found Dataword 0x%.2x\n", (unsigned char)dataBytes[i]);

      for(j = 0; j < 7; j++){
		if ((j + (i*7)) == chain_len) break;

        if (dataBytes[i] % 2 == 0)
             c_from_chip[j + (i*7)] = '0';
        else c_from_chip[j + (i*7)] = '1';

        dataBytes[i] = dataBytes[i] >> 1;
      }
    }
    else if (verbosity > 0) fputs("Received Byte was not a DataByte!\n", stderr);
  }
  
  /* malloc/free-style: free (dataBytes); */

  return 0;
}

/*
int SetPin(int chainID, int chainLength, int pin, int value){
  if (chainID > 4){
    fputs("SetPin() failed! Max 4 chains supported.\n", stderr);
    return -1;
  }
  return 0;
}

int GetPin(int chainID, int chainLength, int pin){
  if (chainID > 4){
    fputs("GetPin() failed! Max 4 chains supported.\n", stderr);
    return -1;
  }
  return 0;
}
*/

int Step(int stepCount, unsigned char chainMask){
  char checksum, hwchecksum, ackbyte;
  char dataBytes[6];
  int i;
  
  if (stepCount > 16383){
    if (verbosity > 0) fputs("Step() failed! Too many steps, max 16383 steps.\n", stderr);
    return -1;
  }

  /* generate instruction byte */
  dataBytes[0] = (char)I_STEP;

  /* generate stepMask bytes */
  dataBytes[1] = (char)((chainMask & 0x0F)       | 0x80); /* low bits  = noScanPath 3-0 */
  dataBytes[2] = (char)(((chainMask >> 4)& 0x0F) | 0x80); /* high bits = ScanPath   3-0 */

  /* generate stepCount bytes */
  dataBytes[3] = (char)((stepCount & 0x7F) | 0x80); /* use lower bits only */
  dataBytes[4] = (char)((stepCount >> 4)   | 0x80); /* use upper bits only */
  
  dataBytes[5] = '\0';

  checksum = (Checksum(dataBytes, strlen(dataBytes)) | 0x80);

/* send instruction */
  InternalWrite(strlen(dataBytes), dataBytes);
  
  if (verbosity > 1){
    printf("\nDEBUG (step): str_len=%d ; db1=0x%.2x ; db2=0x%.2x ; cs=0x%.2x\n\n", strlen(dataBytes), dataBytes[1], dataBytes[2], checksum);
    for (i = 0; i < strlen(dataBytes); i++)
    printf("0x%.2x ", (unsigned char)dataBytes[i]);
    printf("\n\n");
  }


  /* the hardware proceeds its work when receiving the internal calculated checksum */
  InternalWrite(1, &checksum);
  /* get hw-checksum*/
  if (InternalRead(1, &hwchecksum) == -1){
    if (verbosity > 0) fputs("InternalRead() for Instruction Checksum in Step() failed.\n", stderr);
    return -1;
  }  
  if (hwchecksum != checksum){
    if (verbosity > 0) fputs("Step() failed! Checksum error.\n", stderr);
    printf("S: sw=0x%.2x ; hw=0x%.2x \n", checksum, hwchecksum);
    return -1;
  }
  else{
    InternalRead(1, &ackbyte); /*error handling could be realized with timeouts or something*/
    return 0;
  }
}


int Run(unsigned char chainMask){
  char checksum, hwchecksum;
  char dataBytes[4];
  int i;

  /* generate instruction byte */
  dataBytes[0] = (char)I_RUN;

  /* generate stepMask bytes */
  dataBytes[1] = (char)((chainMask & 0x0F)       | 0x80); /* low bits  = noScanPath 3-0 */
  dataBytes[2] = (char)(((chainMask >> 4)& 0x0F) | 0x80); /* high bits = ScanPath   3-0 */

  dataBytes[3] = '\0';

  checksum = (Checksum(dataBytes, strlen(dataBytes)) | 0x80);
  
/* send instruction */
  InternalWrite(strlen(dataBytes), dataBytes);

  if (verbosity > 1){
    printf("\nDEBUG (run): str_len=%d ; db1=0x%.2x ; db2=0x%.2x ; cs=0x%.2x \n\n", strlen(dataBytes), dataBytes[1], dataBytes[2], checksum);
    for (i = 0; i < strlen(dataBytes); i++)
    printf("0x%.2x ", (unsigned char)dataBytes[i]);
    printf("\n\n");
  }
  
  /* get data */
  InternalRead(1, &hwchecksum);
  
  /* the hardware proceeds its work when receiving the same checksum as delivered above */
  InternalWrite(1, &checksum);
  
  if (hwchecksum != checksum){
    if (verbosity > 0) fputs("Run() failed! Checksum error.\n", stderr);
	return -1;
  }
  else return 0;
}


int Stop(){
  char dataBytes[2], response;
  int i;
	
  /* generate instruction byte */
  dataBytes[0] = (char)I_STOP;

  dataBytes[1] = '\0';
  
/* send instruction */
  InternalWrite(strlen(dataBytes), dataBytes);

  if (verbosity > 1){
    printf("\nDEBUG (stop): str_len=%d ; ib=0x%.2x \n\n", strlen(dataBytes), dataBytes[0]);
    for (i = 0; i < strlen(dataBytes); i++)
    printf("0x%.2x ", (unsigned char)dataBytes[i]);
    printf("\n\n");
  }
  
  if (InternalRead(1, &response) == -1){
    if (verbosity > 0) fputs("Stop() failed! Operation timed out.\n", stderr);
	return -1;
  }
  
  if (response != ACK_DONE){
    if (verbosity > 0) fputs("Stop() failed! Incorrect acknowledge received.\n", stderr);
	return -1;
  }
  
  else return 0;
}

